;  coms_ddt.h		Combro		13 February 86
;			ammended	20 September 86
;			ammended	10 January 87
;			ammended	7 August 92

; include file for MASM	/TASM	by Nigel V.Hewitt

; This defines the data structure created in the code and
; refferenced by the routines in coms.obj This file is included
; in the actual assembly of COMS.SYS so it also defines the defaults
; in the distributed version.


; User access to the structures in coms.sys can be made via
; a call to int 14h with ah = 4 and dx = required port
; returns a vector in es:ax to the coresponding structure for that port

; Although the definitions for the ports are sequential please
; allow for the fact that we will add extra items to the end for
; updates so make no assumptions about the structure length for
; later versions ie: use t_size defined here as the length in bytes.
; We will endevour not to change the items defined here.
; All numeric values in this file are specificaly typed hex or decimal
; so you may work ot your prefered default radix


; Definitions of the mode bits

; These bits are set by the user to control the operation of the system
; they are recognised dynamically and most can be changed at any reasonable
; time. (ie. when the buffers are empty and the ports are inactive)

; However : m_norm is only sensed at initialisation time as it controls
; the interupt structure and would cause problems if adjusted live.

m_xtx		equ	0001h	; generate xon/xoff protocols
m_xrx		equ	0002h	; honour any xon/xoff protocols received
m_cts		equ	0004h	; wait the sender till cts is active
m_rts		equ	0008h	; remove rts when the transmitter que is empty
m_cd		equ	0010h	; wait the sender till cd is active
m_norm		equ	0020h	; normal pc/xt/at mode (over-rides all else)
m_dsr		equ	0040h	; wait sender til dsr active
m_dtr		equ	0080h	; remove dtr when receiver is nearly full

; Status bits set to indicate the current status

m_xset		equ	0100h	; we have been told to xoff
m_xsent		equ	0200h	; we have sent xoff
m_setup		equ	0400h	; set by coms_init,subsequent calls will only
;				; adjust the ports normal/extended mode

; Control bits to control timeout modes

m_txto		equ	0800h	; set to enable tx timeouts
m_rxto		equ	1000h	; set to enable rx timeouts

; Error reporting system (parity/over run/framing/break)

m_error		equ	2000h	; record errors in rx que
error_char	equ	0eeh	; error prefix character

; echo receive to transmit option

m_echo		equ	4000h	; special customer request

; 16550 uart flag

m_fifo		equ	8000h	; new version chip detected

; extended mode bits (in emode byte)

m_tms		equ	0001h	; special ENQ/ACK mode
m_force		equ	0002h	; force acceptance of a port address

; Baud rate dividers

b50	equ	2304d	; divisor for 50 bauds
b75	equ	1536d	; etc
b110	equ	1047d
b134	equ	857d
b150	equ	768d
b300	equ	384d
b600	equ	192d
b1200	equ	96d
b2400	equ	48d
b4800	equ	24d
b9600	equ	12d
b19200	equ	6d
b34800	equ	3d
b56000	equ	2d	; aprox 2% error

; Line control bits : the  .line word is made of the sum of those required

; Chose one from each group eg:
;	db8 + osb + nopar	8 bits, one stop bit, no parity
;	db7 + tsb + pare	7 bits, two stop bits, even parity

db5	equ	00h	; 5 data bits
db6	equ	01h	; 6 data bits
db7	equ	02h	; 7 data bits
db8	equ	03h	; 8 data bits

osb	equ	00h	; one stop bit
tsb	equ	04h	; two stop bits

nopar	equ	00h	; no parity generation or checking
pare	equ	18h	; use even parity
paro	equ	08h	; use odd parity
par1	equ	38h	; always send parity as 1
par0	equ	28h	; always send parity as 0



ddt	    struc
base	    dw	0		; port base address
intr	    db	0		; interupt request line to be used
sub_addr    db	0		; port sub address
baud	    dw	b1200		; divider to produce requested baud rate
mode	    dw	m_xtx+m_xrx	; flags (see definitions below)
line	    dw	db8+osb+nopar	; line control register bits
tx_size	    dw	100d		; size in bytes of the transmitter buffer
rx_size	    dw	100d		; size in bytes of the receiver buffer
force	    dw	0		; buffer to hold a xon/xoff protocol character
tx_count    dw	0		; number of bytes in the transmitter buffer
rx_count    dw	0		; number of bytes in the receiver buffer
tx_seg	    dw	0		; segment of tx buffer start
rx_seg	    dw	0		; segment of rx buffer start
tx_off	    dw	0		; offset of tx buffer start
rx_off	    dw	0		; offset of rx buffer start
tx_pointer  dw	0		; pointer to next byte in tx buffer
rx_pointer  dw	0		; pointer to next byte in rx buffer
ier_mask    db	0		; value expected in the int enable reg
pic_mask    db	0		; mask for the pic
time_out_tx dw	64800d		; time out delay transmit
time_out_rx dw	64800d		; time out delay receive
rx_xon	    dw	0		; buffer space for sending xon(0=use half empty)
rx_xoff     dw	20d		; buffer space for sending xoff
p_modem	    db	0	        ; modem control register bits storage
e_mode	    db  0		; dummy place holder to preserve word boundary
e_debug	    dw  0		; used to solve problems (various)
ddt	    ends

t_size	    equ	type ddt	; size of a table entry in bytes

