	page	60,132
;
; MODEM from Combro.			02/11/85 original
;					02/06/87 slight ammendments
;
;			by Nigel V.Hewitt  BSc
;
; This code is supplied in source form with the device drivers
; for the Combro AS4/8 multiple RS232 board. it is included
; not so much as a utility but as an example of using com
; ports to save other users time, hence the code is not optimised
; to give eficient execution and commented heavily.
;
; assemble using IBM or Microsofts MASM or similar and then link
;
;    masm  modem;
;    link  modem;
;
; conditional assembly flags
;
; These should be set to 0 or 1 to generate the kind
; of dumb terminal you want. Read the rest of the code for
; a fuller explanation of each in its turn
;
half		equ	1	; 0=full duplex 1=half duplex
init		equ	0	; initialisation 1=1200 baud,0=leave
crlf		equ	0	; 1=convert cr keys to cr+lf sequence
;
; definitions of useful constants
;
ncom		equ	10	; the maximum com port number
;
cr		equ	0dh	; ascii carrage return
lf		equ	0ah	; ascii line feed
escp		equ	1bh	; ascii escape character
;
; a customer request was to be able to send a "break"
; so I do it on pressing both shift keys, see below
;
right_shift	equ	01h	; code for right hand shift key
left_shift	equ	02h	; .... left hand shift key
ctrl_shift	equ	04h	; .... control shift key
alt_shift	equ	08h	; .... alt shift key
;
;
	include	coms_ddt.h	; include the definition block
;
;
; this is the data segment
; 
data	segment
;
; variables and constants
;
v_page		db	?	; current video page
com		dw	?	; com device to use
;
ten		db	10	; a useful constant
;
ddt_seg		dw	?	; segment address of the ddt
ddt_off		dw	?	; offset address
hardware	dw	?	; port base address
;
; text messages
;
sign_on	db	cr,lf,'MODEM : the simple com port driver'
	db	cr,lf,'by Combro   02/11/85'
	db	cr,lf,lf,'key ESC to terminate session'
	if	init eq 1
	db	cr,lf,'port is initialised to 1200 baud, 8 bit no parity'
	else
	db	cr,lf,'port is expected to be previously initialised'
	endif
	db	cr,lf,'$'
;
bye_bye	db	cr,lf,lf,'Modem terminating$'
;
error	db	cr,lf,'Modem parameter errors'
	db	cr,lf,lf,'enter modem n    where n is the number '
	db	'of the comn: port you wish to use'
	db	cr,lf,'$'
;
data	ends
;
;
;
code	segment
;
	assume	cs:code,ds:data
;
; a simple com port driver: key to line/line to screen
;
modem	proc	far
	sti				; interupts on (normally on anyway)
	mov	ax,seg data		; set up the segment registers
	mov	ds,ax
;
; To get keys to the screen with the minimum of delay I use
; direct bios calls. (consult the IBM technical reference manual
; for more details) this means that I need to know the current video
; page. This is almost invariably zero but...
;
	mov	ah,15			; obtain the current bios video page
	int	10h			; call the bios server
	mov	v_page,bh		; save the current video page
;
; we must anounce our presence to the waiting public
;
	mov	dx,offset sign_on	; address of message
	mov	ah,9			; print a string
	int	21h			; call to dos
;
; find out what port they want to use (1 or 2 digits)
;
; es: still points to the psb which contains the input characters
; if errors are found we branch to k10 which exits with an error message
;
	xor	ah,ah			; we shall use ax as a word later
	mov	bx,5dh			; point to the default fcb
	mov	al,es:[bx]		; get the first character
	sub	al,'0'			; convert to binary
	jb	k1			; to small
	cmp	al,9
	ja	k1			; to big for a digit
	mov	cl,es:[bx]+1		; get the next digit
	cmp	cl,' '			; if this is a space all is well
	je	k2			; ok
	mul	ten			; multiply al by 10 (result in ax)
	sub	cl,'0'			; convert the second digit to binary
	jb	k1			; to small to be a digit
	cmp	cl,9			; test for to big
	ja	k1
	add	al,cl			; gather the bits together in ax
	cmp	byte ptr es:[bx]+2,' '	; the next character must be a space
	je	k2			; it is ok
k1:	jmp	k10			; we don't do 100+ com ports yet
k2:	dec	ax			; convert 1-10 to 0-9
	cmp	ax,ncom			; test the final result is legal
	jnb	k1
	mov	com,ax			; set this as the port to use
;
; if initialisation set the port up
;
; The device drivers (or comset) provide initialisation but optionaly it can
; be provided be provided here by inserting the right values in al.
; (don't forget to ammend the sign on text)
;
	if	init eq 1		; test the conditional flag
	mov	dx,com			; get the port selected
	mov	ah,0			; ah = 0 => set up
	mov	al,083h			; 1200 baud,8 bit,no-parity
	int	14h
	endif
;
; get a pointer to the device definition table in COMS.SYS
; (this is an extention of what the pc bios can do)
;
	mov	dx,com
	mov	ah,4			; special call (see coms_ddt.h)
	int	14h
	mov	ddt_seg,es		; save the returned values
	mov	ddt_off,ax
	mov	bx,ax			; put this in an index reg
	mov	ax,es:[bx].base		; get the base address for the port
	mov	hardware,ax		; save it
;
; this is the start of the loop where we
; hunt for keys and data from the line
;
; we start by testing the keyboard to see if the operator
; has typed anything in
;
k3:	mov	ah,1			; request keyboard status
	int	16h
	jz	k5			; z = no character
	mov	ah,0			; request the character
	int	16h
	cmp	al,escp			; if it is esc we exit
	jz	k9
;
; if the half flag is set at assemble time this code will
; be included so that all keys are echoed on the screen as
; well as being sent : emulating a half duplex terminal
;
	if	half eq 1		; assemble for half duplex
	or	al,al			; beware echoing nulls
	jz	k4			; omit
	push	ax			; save the character
	call	ch_out			; character to screen
	pop	ax			; recover the character
	endif
;
; send the character to the com line
;
k4:	mov	ah,1			; send a character
	mov	dx,com			; com?: value
	int	14h
	jmp	short k3		; back to the start of the loop
;
;
; there may be no keystroke but if the user wants to send a break
; we allow this by presing both shift keys together
;
; I have already called the extended call to get the address of the
; device definition table and extracted from that the base address
; of the actual UART involved. If the user requires a break (constant
; space = 0) then we set the bit in the line control reg while
; he keeps both the left and right shift keys down. If you use
; SideKick or a terminal emulator and shift-shift offends you feel
; free to change it to some other shifts
;
k5:	mov	ah,2			; return shift keys details
	int	16h
	and	al,right_shift+left_shift
	cmp	al,right_shift+left_shift
	jne	k7			; not required
;
	mov	dx,hardware		; obtain the port address
	add	dx,3			; point to the line control register
	in	al,dx
	or	al,40h			; set the break bit
	out	dx,al
;
k6:	mov	ah,2			; return shift keys details
	int	16h
	and	al,right_shift+left_shift
	cmp	al,right_shift+left_shift
	je	k6			; keep it set
;
	mov	dx,hardware		; obtain the port address
	add	dx,3			; point to the line control register
	in	al,dx
	and	al,not 40h		; clear the break bit
	out	dx,al
;
	jmp	short k3		; restart the loop
;
;
; if there are no keystrokes pending then we test the serial
; port to see if it has any data for us to display
;
k7: 	mov	ah,3			; request serial port status
	mov	dx,com			; com?: number
	int	14h
	test	ah,80h			; now IBM and I never set this
	jnz	k8			; bit for status but some
					; compatables do and they then
					; scramble the other bits
	test	ah,1			; data available (dav) ?
	jz	k3			; not now
	mov	ah,2			; request the data
	int	14h
	call	ch_out			; character to screen
	jmp	short k3		; return to the top of the loop
;
; this bit of code realy should not be needed but some makers
; of PC compatible computers (many) have a bug in their bios
; in that it does what IBM says not what IBM does
;
k8:	mov	ah,2			; unscramble a compatable but
	int	14h			; why are you running "modem"
	jmp	short k3		; without the AS4/8 driver?
;
; well the user pressed esc so we shall exit
; back to dos. As all went well we shall set errorlevel 0
;
k9:	mov	dx,offset bye_bye	; sign off message
	mov	ah,9			; print a string
	int	21h
;
	mov	ah,4ch			; exit to dos
	mov	al,0			; return code
	int	21h
;
; we only get here if the user has entered an illegal
; parameter string when he was supposed to give the
; com port number
;
k10:	mov	dx,offset error		; error message
	mov	ah,9			; text to screen
	int	21h
;
	mov	ah,4ch			; exit to dos
	mov	al,1			; set errorlevel = 1
	int	21h
;
modem	endp
;
;
; this is a procedure to output a character to the screen
; I could use a dos service call in each case but as I have need
; to expand the operation and as redirection could give a heavy overhead
; I chose not to
;
ch_out	proc	near			; output a character
;
; some terminals on receiving a carrage return (cr) only return the
; cursor to the left hand screen edge and need to be sent a
; line feed (lf) to cause them to move to the line below. others will
; automatically insert a lf with any cr received. some do both
; and provide a switch option to select the desired mode as I do here.
;
	if	crlf eq 1		; assemble if cr => cr + lf option
	cmp	al,cr			; test if this is a cr
	jnz	ch1			; jump if it is not
	mov	bh,v_page		; select the current page
	mov	ah,14			; screen drivers tty mode
	int	10h
	mov	ax,cr			; reload al with a lf
	endif
;
ch1:	mov	bh,v_page		; load the current page
	mov	ah,14			; tty mode
	int	10h
	ret
;
ch_out	endp
;
;
code	ends
;
;
; it remains only to allocate some stack for the program:
; the device drivers are interupt driven and can break in at any time
; but they leave things as they find them but the require not more than
; 11 words of stack to work in. But why be stingy? IBM recomend 100 words
; to cover all nested interupts etc so ....
;
sseg	segment	stack
	dw	100 dup (?)
sseg	ends
;
	end	modem		; the start of execution address
